#include <opencv2/opencv.hpp>
#include <iostream>
#include <string>
#include <stdio.h>

using namespace cv;
using namespace std;

void paint(Mat& dst, Mat& src, int height, int width, int BrushSize, int levels) {
	int* CountIntensity = new int[levels];
	int* RedAverage = new int[levels];
	int* GreenAverage = new int[levels];
	int* BlueAverage = new int[levels];

	for (int y = BrushSize; y < (height - BrushSize); y++)
	{
		for (int x = BrushSize; x < (width - BrushSize); x++)
		{
			//cout << "y " << y << " x " << x << endl;
			// initialize the array
			for (int i = 0; i < levels; i++)
			{
				CountIntensity[i] = 0;
				RedAverage[i] = 0;
				GreenAverage[i] = 0;
				BlueAverage[i] = 0;
			}

			for (int j = -BrushSize; j <= BrushSize; j++)
			{
				for (int i = -BrushSize; i <= BrushSize; i++)
				{
					//cout << "y+j " << y+j << " x+i " << x+i << endl;
					double b = src.at<cv::Vec3b>(y + j, x + i)[0];
					double g = src.at<cv::Vec3b>(y + j, x + i)[1];
					double r = src.at<cv::Vec3b>(y + j, x + i)[2];
					//cout << "y+j " << y+j << " x+i " << x+i << " " << b << " " << g << " " << r << endl;

					int nCurIntensity = (((r + g + b) / 3.0) * levels) / 255;
					//cout << "y+j " << y+j << " x+i " << x+i << " " << nCurIntensity << endl;

					int tmp = nCurIntensity;
					CountIntensity[tmp]++;

					BlueAverage[tmp] = BlueAverage[tmp] + b;
					GreenAverage[tmp] = GreenAverage[tmp] + g;
					RedAverage[tmp] = RedAverage[tmp] + r;
					//cout << "Average " << RedAverage[tmp] << endl;
				}
			}

			int nCurMax = 0;
			int nMaxIndex = 0;
			for (int nI = 0; nI < levels; nI++)
			{
				if (CountIntensity[nI] > nCurMax)
				{
					nCurMax = CountIntensity[nI];
					nMaxIndex = nI;
				}
			}
			//cout << "CurMax " << nCurMax << " MaxIndex " << nMaxIndex << endl;

			double tmpB = BlueAverage[nMaxIndex] / nCurMax;
			double tmpG = GreenAverage[nMaxIndex] / nCurMax;
			double tmpR = RedAverage[nMaxIndex] / nCurMax;
			Vec3b pixelColor(tmpB, tmpG, tmpR);
			dst.at<Vec3b>(y, x) = pixelColor;
			//dst.at<Vec3b>(y,x)[0] = BlueAverage[nMaxIndex] / nCurMax;
			//dst.at<Vec3b>(y,x)[1] = GreenAverage[nMaxIndex] / nCurMax;
			//dst.at<Vec3b>(y,x)[2] = RedAverage[nMaxIndex] / nCurMax;
		}
	}
}

int main(int argc, char* argv[]) {
	if (argc != 4) {
		std::cout << argv[0] << ": invalid number of arguments\n";
		return 1;
	}
	int brushSize = std::atoi(argv[1]);
	int levels = std::atoi(argv[2]);
	std::string fileName = argv[3];

	Mat src = imread(fileName, 1);
	//namedWindow("src", WINDOW_KEEPRATIO);
	imshow("src", src);
	Mat dst = Mat(src.size(), CV_8UC3);

	unsigned long AAtime = 0, BBtime = 0; //check processing time
	AAtime = getTickCount(); //check processing time

	paint(dst, src, src.rows, src.cols, brushSize, levels);

	BBtime = getTickCount(); //check processing time
	printf("%.2lf sec \n", (BBtime - AAtime) / getTickFrequency()); //check processing time

	//namedWindow("dst", WINDOW_KEEPRATIO);
	imshow("dst", dst);

	vector<int> compression_params;
	compression_params.push_back(IMWRITE_PNG_COMPRESSION);
	compression_params.push_back(9);
	imwrite("oilVersion-"+fileName, dst, compression_params);

	destroyAllWindows();
	return 0;
}
